/*-
 * Copyright (c) 2008 Andrey V. Surkov <nsand@sura.ru>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	 $Id: db.c,v 0.52 2008/05/23 07:54:36 nsand Exp $
 */

#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <unistd.h>

#include <netinet/in.h>
#include <netinet/in_systm.h>
#include <netinet/ip.h>
#include <netinet/ip_var.h>
#include <netinet/tcp.h>

#include "db.h"

extern	uint32_t MAXTHREADS;
extern	uint32_t countpks;
extern	struct pks *db;		 /* buffer for storage current streams */
extern	struct filter *filterdb; /* storage for filtration file */

/* 
*  Search empty space in DB, if dst == 0 then space is empty 
*/
u_long
FindEmptyPlaceDB(void)
{
    u_long i;
    for (i = 0; i < MAXTHREADS; i++)
	if (db[i].dst == 0){
	    return i;
	}
    return MAXTHREADS;
}


/*
*  Adding stream in DB (HTTP 200,206)
*
*/
u_long 
AddPKSDB(const char * packet)
{

    struct ip *ip 	= (struct ip *) (packet); 
    struct tcphdr *tcp	= (struct tcphdr *)(packet +  (ip->ip_hl<<2));

    /* if not empty place in DB return MAXTHREADS */
    u_long npks = FindEmptyPlaceDB();
    if (npks >= MAXTHREADS) 	{Logger("Increase MAXTHREADS\n");return MAXTHREADS;}

    /* fill all value in structure */
    ClearPKS(npks);
    db[npks].start_count = ntohl(tcp->th_seq);
    db[npks].dst 	= ip->ip_dst.s_addr;
    db[npks].src 	= ip->ip_src.s_addr;
    db[npks].port_d 	= tcp->th_dport;
    db[npks].port_s 	= tcp->th_sport;
    db[npks].ack	= tcp->th_ack;
    db[npks].lencur 	= 0;
    db[npks].len 	= 0;
    db[npks].X 		= 0;
    db[npks].httpf 	= 0;
    time(&db[npks].atime);
    time(&db[npks].ctime);
    sprintf(db[npks].name,"%08x_%08x_%04x_%04x_%08x-%x.",
	db[npks].dst, db[npks].src,db[npks].port_s, db[npks].port_d, db[npks].ack,npks);
    
    /* increase count stream in DB */
    countpks++;	
    
    /* return refer on current stream */
    return npks;
}


/*
* Search in DB such stream as current, if found refer to him
*/
u_long
SearchOldNameDB(u_long numpks)
{
    u_long i;
    for (i = 0;i < MAXTHREADS; i++)
	if (db[i].refer  == MAXTHREADS)		//     
	if (strcmp(db[i].etag,db[numpks].etag) == 0)
	if (db[i].dst 	  == db[numpks].dst)
//	if (db[i].src 	  == db[numpks].src)	//    
//	if (db[i].port_s == db[numpks].port_s)	//        etag      
	if (i != numpks)
	if (db[i].len    == db[numpks].len)
	if (strcmp(db[i].findname,db[numpks].findname) == 0)
	{
	    return i;
	}
return MAXTHREADS;
}


/*
*  Search stream in DB for incoming packet, return refer to stream if found
*/
u_long
FindPKSDB(const char *packet)
{
    struct ip *ip 	= (struct ip *) (packet); 
    struct tcphdr *tcp	= (struct tcphdr *)(packet +  (ip->ip_hl<<2));
    u_long i = 0;
    for (i = 0; i < MAXTHREADS; i++) {
    	if (db[i].ack == tcp->th_ack)
	if (db[i].dst == ip->ip_dst.s_addr)
	if (db[i].src == ip->ip_src.s_addr)
	if (db[i].port_d == tcp->th_dport)
	if (db[i].port_s == tcp->th_sport)
	{ 
	    return i;
	}
    }
    return MAXTHREADS;
}


/*
*  Get X (offset) to position in file from title Content-Range: X-Y/Z
*/
u_int64_t
GetX(char *data_tcp)
{
    u_int64_t X=0;
    char * found_str=strcasestr(data_tcp,"Content-Range: bytes");
    if (found_str != NULL) {
	sscanf(found_str+21,"%llu",&X);
	return X;
    }
return 0;
}


/*
*	Get file length from title Content-Range: or Content-Length:
*/
u_int64_t
FileLength(const char * data_tcp)
{
    u_int64_t filelength = 0;

    char * found_str = strcasestr(data_tcp,"Content-Range: bytes");
    if (found_str != NULL) {
	char *found_str2 = strstr(found_str + 21,"/");
	if (found_str2 != NULL)
	    if (sscanf(found_str2 + 1,"%llu",&filelength) > 0) {
	return filelength;} 
    }
    found_str = strcasestr(data_tcp,"Content-Length:");
    if (found_str != NULL) {
	if (sscanf(found_str + 15,"%llu",&filelength) > 0) {
	    return filelength;
	} else { 
	Logger("not found Content-Length:\n");
	}
    }
return 0;
}


/*
*  clear stream in DB (PKS)
*/
void 
ClearPKS(u_long ref)
{
    bzero(&db[ref],PKS);
    db[ref].refer = MAXTHREADS;
    db[ref].fd = -1;
}


/*
* check the file by type and size before add in DB
*/
int
Filter(u_long ref)
{
    int i;
    for (i = 0; filterdb[i].type[0]>0; i++)
	if (strcmp(filterdb[i].type,db[ref].type) == 0 && db[ref].type[0] > 0)
	    {
	    if (db[ref].len < filterdb[i].min ||  db[ref].len > filterdb[i].max) return 1;
	    }
return 0;
}

