/*
 * Copyright (c) 2008 Andrey V. Surkov <nsand@sura.ru>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	 $Id: interface.c,v 0.58 2008/05/23 07:54:36 nsand Exp $
 */
 
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/stat.h>
#include <stdio.h>

#include <netinet/in.h>
#include <netinet/in_systm.h>

#include <ncurses.h>
#include <panel.h>
#include <math.h>

#include "interface.h"
#include "skyfish.h"

#define SETCOLOR(a) wattrset(win,COLOR_PAIR(a))
#define FCOLOR(w,a) wattrset(w,COLOR_PAIR(a))

int 	V_LINE;				/* heigth window */
int 	H_LINE;				/* width window	*/
int 	V_CURSOR;			/* position in DB */
int 	DEL_FILE;			/* flag is mark to delete file */
u_char KEYPRESS;			/* has been pressed key  */
u_long swinfo;				/* switch file info */
u_long swhide;				/* switch to show hidden streams */

 WINDOW *win;
 WINDOW *winfilter;
 PANEL *panel;
 PANEL *winpanel;

int winfilteron;
int winbottom;
extern int done;
extern  u_long MAXTHREADS;			//  - 
u_long realspeed;
extern      u_long countpks;
extern      u_long discount;
u_int64_t alltraf; 
u_int64_t writraf;     
extern     struct pks *db;
extern     struct filter *filterdb;
char tmp3 [320];
extern     char src[];
extern     char dst[];
extern     char tmp[];
extern     u_long frpks;
u_long swmas;
u_char filtervisible;
u_long Fx,Fy,Fs;
u_int64_t Fk;
u_long Ps;
u_long Py;
u_long Ph; 	/* count of passed streams in buffer, don't show in window */
u_long Poff;
u_long mesto;

char bufconf[0xffff];

void
show_table(void)
{
    u_long i;
    u_long num;

    if (swmas)	ShowMas();
    showhead();		/* title programm */
    show_statistics();	/* statistics open files, write */
    ShowHeadTable();	/* header table */

    /* clear window to bottom */
    SETCOLOR(3);
    wmove(win,6, 0);	
    wclrtobot(win);

    if (db[V_CURSOR].refer != MAXTHREADS || db[V_CURSOR].dst == 0)
        { IncreaseCursor(); DecreaseCursor(); }
    Ph=0;
    Ps=0;
    num=1;
    mesto = 1;
    for (i=0; i < MAXTHREADS;i++) {
	if (db[i].dst > 0 && db[i].refer == MAXTHREADS && db[i].httpf == 1 && db[i].begin ==1)
	    {if (V_CURSOR == i) mesto = num;
	    num++;	    }
	}
    
    if (mesto > (V_LINE - 15 + winbottom)) Ps= mesto - (V_LINE - 15 + winbottom);
    
    Poff=0;
    num=1;
    for (i=0; i < MAXTHREADS && Poff < (V_LINE-15 + winbottom);i++) {
	if ((db[i].dst > 0 && db[i].refer == MAXTHREADS && db[i].httpf == 1 && db[i].begin ==1)
	    || (swhide == 1 && db[i].dst > 0))  {
	    
	    	if (num > Ps ) {
		    Poff++;
		    CreateStringStream(i,num);
		    wmove(win,5 + Poff,0);
		    PrintColorString(i);
		    CheckMarkDel(i);
		}
		num++;
	}
    } 	/* for */
 
    KEYPRESS = 0;
    if (swinfo  == 1) { ShowInfoFile();}
    if (filtervisible) ChangeFilter();
    update_panels();
    doupdate();
}


/*
*
*/
int
init_scr(void)
{
    initscr();
    if(!has_colors()) {
	endwin();
	return 1;
    }
    if(start_color() != OK) {
	endwin();
	return 1;
    }

    cbreak();
    noecho();
    nodelay(stdscr,TRUE);
    keypad(stdscr,TRUE);
    raw();
    
    init_pair(1, COLOR_WHITE, COLOR_BLACK);
    init_pair(2, COLOR_WHITE, COLOR_BLUE);
    init_pair(3, COLOR_WHITE, COLOR_BLACK);
    init_pair(4, COLOR_BLACK, COLOR_WHITE);
    init_pair(5, COLOR_BLACK, COLOR_YELLOW);
    init_pair(6, COLOR_WHITE, COLOR_RED);
    init_pair(7, COLOR_WHITE, COLOR_BLUE);
    init_pair(8, COLOR_BLACK, COLOR_WHITE);
    init_pair(9, COLOR_GREEN, COLOR_BLACK);
    init_pair(10, COLOR_BLACK, COLOR_GREEN);
    
    
    win = newwin(0,0, 0, 0);
    wbkgdset(win, COLOR_PAIR(1));
    wclear(win);
    wrefresh(win);

    H_LINE = getmaxx(win);
    V_LINE = getmaxy(win);

    winfilter = newwin(V_LINE-6,H_LINE-10, 5,5);
    wbkgdset(winfilter, COLOR_PAIR(10));
    wclear(winfilter);
    wrefresh(winfilter);
    box(winfilter,0,0);
    wprintw(winfilter,"For testing only!");
    scrollok(winfilter, TRUE);
	
    panel = new_panel(winfilter);
    hide_panel(panel);
    winpanel = new_panel(win);
	
    update_panels();
    doupdate();
    filtervisible=0;
    winfilteron = 0;
    swinfo	= 0;
    winbottom = 0;
    Fy=2;
    return 0;
}


/*
* show statistics data line
*/
void
show_statistics(void)
{
	wmove(win,3,24);
	SETCOLOR(1);	wprintw(win," IN:");
	SETCOLOR(9);	PrintCount(win,alltraf);
	SETCOLOR(1);	wprintw(win," Write:");
	SETCOLOR(9);	PrintCount(win,writraf);
	wmove(win,3,61 + H_LINE - 80);	wprintw(win,"  press 'Q' to exit"); 
	SETCOLOR(1);
//	wmove(win,4,0);		wprintw(win,"CountPKS = %u",countpks);
	wmove(win,4,0);		wprintw(win,"CountPKS = %u",countpks);
	wmove(win,4,15);	wprintw(win,"OpenFile = %u",Getopenfiles());
	wmove(win,4,30);	wprintw(win,"Bad = %lu",discount);	
//	wmove(win,4,40);	wprintw(win,"Vc = %lu",V_CURSOR);		
//	wmove(win,4,48);	wprintw(win,"Ps = %lu",Ps);			
//	wmove(win,4,56);	wprintw(win,"Ph = %lu",Ph);
//	wmove(win,4,63);	wprintw(win,"mes= %lu",mesto);	
//	wmove(win,4,45);	wprintw(win,"Frpks = %u",frpks);		
}


/*
*  
*/
void
ShowSpeed(void)
{
    int x,y;
    getyx(win,y,x);

    wmove(win,3,0);
    if (realspeed < 1024) {
    SETCOLOR(6); }	else SETCOLOR(1);
    wprintw(win,"Symbol rate:");
    SETCOLOR(9);	PrintSpeed(realspeed);

    wmove(win,y,x);
}


void
CheckKey(int ch)
{
	switch (ch) {
	case 'g':
	case 'G': 	Fvisible(); break;
	case KEY_UP: 	if (filtervisible) Fup(); else DecreaseCursor(); KEYPRESS = 1; break;
	case KEY_DOWN: 	if (filtervisible) Fdown(); else IncreaseCursor(); KEYPRESS = 1; break;
	case KEY_LEFT: 	if (filtervisible) Fleft(); break;
	case KEY_RIGHT:	if (filtervisible) Fright(); break;
	case KEY_DC: 	if (filtervisible) Fk=4; else DEL_FILE = 1; show_table(); CheckTime(); break;
	case KEY_PPAGE: if (filtervisible) Fk=2; else PuCursor(); break;	// Page Up
	case KEY_NPAGE: if (filtervisible) Fk=1; else PdCursor(); break;	// Page Down
	case 'F':
	case 'f': 	ChFastScr(); 	break;
	case 'i': 	SwitchInfo(); 	break;
	case 'w': 	SwitchMas(); 	break;	
	case 'h': 	ShowHide(); 	break;	
	case 'r': 	clearok(win,TRUE); break;	
	case 'v': 	GWindow();	break;
	case 'd': 	ShowFilter(); 	break;		
	case 'Q': 	done = 0; 	break;
	case '0':	Fk=3;		break;
	case '1':	Fk=100*KBYTEL; 	break;
	case '2': 	Fk=MBYTEL; 	break;
	case '3':	Fk=10*MBYTEL; 	break;
	case '4':	Fk=100*MBYTEL; 	break;
	case '5':	Fk=GBYTEL; 	break;
	case '6':	Fk=10*GBYTEL; 	break;
	case '7':	Fk=100*GBYTEL;	break;
	case '8': 	Fk=TBYTEL; 	break;
	case '9': 	Fk=10*TBYTEL; 	break;
	case 'S': 	SaveConfig(); 	break;
	}
    show_table();
}


void 
DecreaseCursor(void)
{
    int i;
    if (V_CURSOR > 0) 
	for (i = V_CURSOR - 1;i >= 0; i--)
	    if ((db[i].refer == MAXTHREADS && db[i].dst > 0 && db[i].lencur > 50 && db[i].begin == 1)  //{// && db[i].type[0] != 0)  { //{ //&& db[i].begin == 200  && db[i].type[0] != '1') { 
	    	|| (swhide == 1 && db[i].dst > 0))  {
		V_CURSOR = i; break;
	    }
}


void
IncreaseCursor(void)
{
    int i;
    for (i = V_CURSOR + 1;i < MAXTHREADS; i++)
	if ((db[i].refer == MAXTHREADS && db[i].dst > 0 && db[i].lencur > 50 && db[i].begin == 1)  //{// && db[i].type[0] != 0)  { //{ //&& db[i].begin == 200  && db[i].type[0] != '1') { 
		|| (swhide == 1 && db[i].dst > 0))  {
	    V_CURSOR = i; break;
	}
}


void 
PuCursor(void)
{
    int i;
    for (i = 0; i < 20; i++)
	DecreaseCursor();
}


void 
PdCursor(void)
{
    int i;
    for (i = 0; i < 20; i++)
	IncreaseCursor();
}


void
PrintSpeed(u_int64_t number)
{
	     if (number > GBYTEL) PrintGBs(number);
	else if (number > MBYTEL) PrintMBs(number);
			 else PrintKBs(number);
}


void
PrintCount(WINDOW *w,u_int64_t number)
{
	     if (number > TB) PrintTB(w,number);
	else if (number > GB) PrintGB(w,number);
	else if (number > MB) PrintMB(w,number);
			 else PrintKB(w,number);
}


void
PrintKBs(u_int64_t number)
{
    wprintw(win,"%6.2f[Kb/s]",number/KBYTE);
}


void
PrintMBs(u_int64_t number)
{
    wprintw(win,"%6.2f[Mb/s]",number/MBYTE);
}


void
PrintGBs(u_int64_t number)
{
    wprintw(win,"%6.2f[Gb/s]",number/GBYTE);
}


void
PrintKB(WINDOW* w,u_int64_t number)
{
    wprintw(w,"%6.2f[Kb]",number/KBYTE);
}


void
PrintMB(WINDOW* w,u_int64_t number)
{
    wprintw(w,"%6.2f[Mb]",number/MBYTE);
}


void
PrintGB(WINDOW* w,u_int64_t number)
{
    wprintw(w,"%6.2f[Gb]",number/GBYTE);
}


void
PrintTB(WINDOW* w,u_int64_t number)
{
    wprintw(w,"%6.2f[Tb]",number/TBYTE);
}


void
showhead(void)
{
	SETCOLOR(2);
	wmove(win,0, 0);			whline(win,ACS_HLINE,H_LINE);
	wmove(win,1, 0);			whline(win,' ',H_LINE); 
	wmove(win,1, H_LINE / 2 - 7);	wprintw(win,"SkyFish %s",VERSION);
	wmove(win,2, 0);			whline(win,ACS_HLINE,H_LINE);
	SETCOLOR(1);
	wmove(win,3, 24);			whline(win,' ',H_LINE - 24);
	wmove(win,4, 0);			whline(win,' ',H_LINE); 
}


void
ShowHeadTable(void)
{
	SETCOLOR(4);
	wmove(win,5, 0);		whline(win,' ',H_LINE);
	wmove(win,5, 0);		wprintw(win,"|#N |          Filename");
	wmove(win,5,57 + H_LINE - 80); 	wprintw(win,"|type | Lenght|Current|");
	SETCOLOR(1);			wprintw(win," ");
}


void
GWindow(void)
{
    if (winfilteron == 0) {
    	winfilteron = 1;
	show_panel(panel);
    } else {
        hide_panel(panel);
	winfilteron = 0;
    }
}


void
SwitchInfo(void)
{
    if (swinfo != 1) { swinfo = 1; winbottom = 0;}
    else {swinfo = 0; winbottom = 9;}
}


void
SwitchMas(void)
{
    if (swmas != 1) swmas = 1;
    else swmas = 0;
}


void
ShowHide(void)
{
    if (swhide != 0) swhide = 0;
    else swhide = 1;
}


void
ShowInfoFile(void)
{

    int i;
//    time_t t;
//    time(&t);

    wmove(win,V_LINE - 9,0);
    wclrtobot(win);
    u_long sumref=CountRefer(V_CURSOR);
    if (db[V_CURSOR].dst !=0) {
    	wprintw(win,"File: ");
//	wprintw(win,"httpf=%1d",db[V_CURSOR].httpf);
//	wprintw(win," lenHttp=%-5u",db[V_CURSOR].http_offset);
	wprintw(win," curlen=%-10llu",db[V_CURSOR].lencur);
//	wprintw(win," type=%-5s",db[V_CURSOR].type);
//	wprintw(win," ref=%-3u",db[V_CURSOR].refer);
	wprintw(win," len=%-10llu",db[V_CURSOR].len);
//	wprintw(win," Numfx=%-10lu",db[V_CURSOR].numfx);
	wprintw(win," Numref=%-10lu",sumref);
	wprintw(win," sta=%-10lu",db[V_CURSOR].start_count);
	wprintw(win," stop [%-lu]sec",db[V_CURSOR].deltatime);
	wprintw(win,"\n%s",db[V_CURSOR].findname);	
	SETCOLOR(9);
//	if (swinfo  == 1) PrintHTTP(V_CURSOR);
	PrintHTTP(V_CURSOR);
    }
    wmove(win,V_LINE,H_LINE);
}


void
PrintHTTP(u_long refer)
{
    u_long p = V_LINE - 7;
    u_long r=0,n=0;
    u_long end = strlen(db[refer].http);
    for (r = 0; r < end; r++) {
	wmove(win,p,n);
	waddch(win,db[refer].http[r]);
	n++;
	if (n >= H_LINE) {n=0;p++;}
	}
}


/*
*  how many refers to file
*/
u_long 
CountRefer(u_long refer)
{
    u_long sum=0;
    u_long i;
    for (i = 0; i < MAXTHREADS; i++) {
	if (db[i].refer == refer) {
	    sum++;
	}
    }
return sum;
}


void ShowMas(void)
{
    u_long i;
    wmove(winfilter,0,0);
    wclrtobot(winfilter);
    wprintw(winfilter,"Findname=%s",db[V_CURSOR].findname);
    wmove(winfilter,2,0);
    wprintw(winfilter,"Etag: %s",db[V_CURSOR].etag);
    wmove(winfilter,3,0);
    for (i=0; i<db[V_CURSOR].numfx; i+=2)
	wprintw(winfilter,"[%llu - %llu] ",db[V_CURSOR].mas[i],db[V_CURSOR].mas[i+1]);
}


void
ShowFilter(void)
{
    wmove(winfilter,0,0);
    wprintw(winfilter,"num\ttype\tmin\tmax");
    wmove(winfilter,1,0);
    u_long i;
    for (i=0; filterdb[i].type[0] > 0; i++)
	wprintw(winfilter,"%u\t%s\t%llu\t%llu\n",i,filterdb[i].type,filterdb[i].min,filterdb[i].max);
}

void
Fvisible(void)
{
    if (filtervisible == 0) filtervisible = 1;
    else filtervisible = 0;

}


/* show window filter to change it */
void
ChangeFilter(void)
{
    WINDOW* w = win;
    u_long i;
    int X=H_LINE-30,Y=6;

    DrawLine(w,X,Y);
    
    for(i=0; filterdb[Fs+i].type[0]>0 && (i<V_LINE-6); i++)
    {
	GShowType(w,X,Y,i);
	Gshowmin(w,X,Y,i);
	Gshowmax(w,X,Y,i);
	if (filterdb[Fs+i].min > filterdb[Fs+i].max) filterdb[Fs+i].max=filterdb[Fs+i].min;
    }
    
Fk=0;	    
}


void
Fup(void)
{
    if (Fy > 0) Fy--;
    else if (Fs > 0) Fs--;
}

void
Fdown(void)
{
    if (Fy < V_LINE-7) Fy++;
    else if (filterdb[Fs+V_LINE-6].type[0] > 0) Fs++;
}


/*
* move cursor in filter window
*/
void
Fleft(void)
{
    if (Fx > 0) Fx--;
}


/*
* move cursor in filter window
*/
void
Fright(void)
{
    if (Fx < 2) Fx++;
}


u_int64_t
Filterminus(u_int64_t num)
{
    u_int64_t delta = num * 0.1;
    if (num <= 512) return 0;
    if (num > delta) return (num -= delta);
return 0;
}


u_int64_t
Filterplus(u_int64_t num)
{
    u_int64_t delta = num * 0.1;
    if (num == 0) return 512;
    if (num < 10*TBYTEL) return (num += delta);
return 10*TBYTEL;
}


void
DrawLine(WINDOW *w,u_long X,u_long Y)
{
    FCOLOR(w,9);
    wmove(w,Y,X);
    wvline(w,ACS_VLINE,V_LINE-6);
    wmove(w,Y,X+7);
    wvline(w,ACS_VLINE,V_LINE-6);
    wmove(w,Y,X+18);
    wvline(w,ACS_VLINE,V_LINE-6);
    wmove(w,Y,H_LINE-1);
    wvline(w,ACS_VLINE,V_LINE-6);
    
    	FCOLOR(w,4);
	wmove(w,Y-1,X);
	wprintw(w,"|Type  | Minimum  | Maximum  |");

} 


/* show column type */
void
GShowType(WINDOW *w,int X,int Y,u_long i)
{
        wmove(w,Y+i,X+1);
        if (Fy == i && Fx == 0) {FCOLOR(w,5);
        if (Fk==4) {filterdb[Fs+i].min=0; filterdb[Fs+i].max=0;}
        } else FCOLOR(w,9);
        wprintw(w,"%-6s",filterdb[Fs+i].type);
}


/* show column minimum */    
void
Gshowmin(WINDOW *w,int X,int Y,u_long i)
{
        if (Fy == i && Fx == 1) {FCOLOR(w,5);
    	if (Fk==1) { filterdb[Fs+i].min=Filterminus(filterdb[Fs+i].min);}	    
    	if (Fk==2) { filterdb[Fs+i].min=Filterplus(filterdb[Fs+i].min);}
    	if (Fk==3) filterdb[Fs+i].min=0;
	if (Fk>=10) filterdb[Fs+i].min=Fk;
	if (Fk==4) {filterdb[Fs+i].min=0; filterdb[Fs+i].max=0;}
	    } else FCOLOR(w,9);
        wmove(w,Y+i,X+8);
	if (filterdb[Fs+i].min == 0) wprintw(w,"%10s","---");
        else PrintCount(w,filterdb[Fs+i].min);
}


/* show column maximum */    
void
Gshowmax(WINDOW *w,int X,int Y,u_long i)
{
        if (Fy == i && Fx == 2) {FCOLOR(w,5);
	if (Fk==1) {filterdb[Fs+i].max=Filterminus(filterdb[Fs+i].max);}		
	if (Fk==2) filterdb[Fs+i].max=Filterplus(filterdb[Fs+i].max);
	if (Fk==3) filterdb[Fs+i].max=0;
	if (Fk>=10) filterdb[Fs+i].max=Fk;
	if (Fk==4) {filterdb[Fs+i].min=0; filterdb[Fs+i].max=0;}
	if (filterdb[Fs+i].max < filterdb[Fs+i].min) filterdb[Fs+i].min=filterdb[Fs+i].max;
	} else FCOLOR(w,9);
	wmove(w,Y+i,X+19);	    		
	if (filterdb[Fs+i].max == 0) wprintw(w,"%10s","---");
	else PrintCount(w,filterdb[Fs+i].max);		
}


/*
* read old file in memory, then delete all filter type and save with new filter type
*/
int
SaveConfig(void) 
{
    FILE *fc;
    char s[220];
    u_long sum=0;
    u_long i;
    u_long offset=0;
    u_long len;
    u_long begin=0;
    char stext[0xffff];
    char *posbuf = &bufconf[0];
char *postext = &stext[0];
    if ((fc = fopen("./skyfish.conf","r")) == NULL){
        if ((fc = fopen("/usr/local/etc/skyfish.conf","r")) == NULL){
	    printf("Not found file skyfish.conf\n");
	    return 1;
	}
    }
	bzero(&bufconf[0],sizeof(bufconf));
	bzero(&stext[0],sizeof(stext));
    fread(bufconf,1,0xffff,fc);
    fclose(fc);

    /* modify filter delete 'type' string from bufferconf */

for(;*posbuf>0;)
{

	if (*posbuf == 't' && *(posbuf + 1) == 'y' && *(posbuf + 2)=='p' && *(posbuf + 3)=='e')
		{ 
		for (;*posbuf!=0xA;posbuf++);
		posbuf++;
	} else 
	if (*posbuf == 'e' && *(posbuf + 1) == 'n' && *(posbuf + 2)=='d')
		{ 
		for (;*posbuf!=0xA;posbuf++);
		posbuf++;
	} 
	else {
		*postext = *posbuf;
		posbuf++;
		postext++;
	}	
}

	/* add type in stext */
    for (i=0; i<filterdb[i].type[0]>0; i++)
	{
	bzero(&s[0],sizeof(s));
	sprintf(s,"type %6s",filterdb[i].type);	
	strcat(stext,s);
	
	bzero(&s[0],sizeof(s));
	if (filterdb[i].min >= GBYTEL) 		sprintf(s,"%10.0fGb",filterdb[i].min/GBYTE);	
	else if (filterdb[i].min >= MBYTEL) 	sprintf(s,"%10.0fMb",filterdb[i].min/MBYTE);
	else 					sprintf(s,"%10.0fKb",filterdb[i].min/KBYTE);
	strcat(stext,s);

	bzero(&s[0],sizeof(s));
	if (filterdb[i].max >= GBYTEL) 		sprintf(s,"%10.0fGb\n",filterdb[i].max/GBYTE);	
	else if (filterdb[i].max >= MBYTEL) 	sprintf(s,"%10.0fMb\n",filterdb[i].max/MBYTE);
	else    				sprintf(s,"%10.0fKb\n",filterdb[i].max/KBYTE);
	strcat(stext,s);
	
}	

	strcat(stext,"end");

    /* save new config file */
    if ((fc = fopen("./skyfish.conf","w")) == NULL){
	    printf("Not found file skyfish.conf\n");
	    return 1;
    }
    fwrite(stext,1,strlen(stext),fc);
    fclose(fc);
    return 0;
}


void
CloseWindows(void)
{
    del_panel(panel);
    delwin(win);
    delwin(winfilter);
    endwin();
}


void 
Resize(int sig_no)
{
    endwin();
    refresh();
    H_LINE = getmaxx(win);
    V_LINE = getmaxy(win);
}


void
SetRealspeed(u_long s)
{
    realspeed = s;
}

void
AddAlltraf(u_long lenip)
{
    alltraf += lenip;
}

void
SetAlltraf(u_long n)
{
    alltraf = n;
}

void
DelWritraf(u_int64_t len)
{
    writraf -=len;
}


void
AddWritraf(u_int64_t len)
{
    writraf +=len;
}

void
SetWritraf(u_int64_t len)
{
    writraf = len;
}


void
CreateStringStream(u_long i, u_long num)
{
    u_int64_t traf;
    char tmp2 [320];
//    char tmp3 [320];
 
	    bzero(&tmp2[0],320);
	    bzero(&tmp3[0],320);
	    strncpy(tmp2,db[i].name,H_LINE - 28);
	    sprintf(tmp3,"|%3d|",num); //   -  
	    strncat(tmp3,tmp2,H_LINE - 28);
	    memset(tmp2,32,320);
	    strncat(tmp3,tmp2,H_LINE - 23 - strlen(tmp3));
	    memset(tmp2,0,320);
	    sprintf(tmp2,"|%-5s",db[i].type);
	    strncat(tmp3,tmp2,strlen(tmp2));
	    bzero(&tmp2[0],320);
	    traf=db[i].len;
	    if (traf > GB) 		sprintf(tmp2,"|%6.2fG|",traf/GBYTE);	
	    else if (traf > MB) 	sprintf(tmp2,"|%6.2fM|",traf/MBYTE);
	    else	 		sprintf(tmp2,"|%6.2fK|",traf/KBYTE);
	    strcat(tmp3,tmp2);	
	    bzero(&tmp2[0],320);
	    traf=db[i].lencur;
	    if (traf > GB) 		sprintf(tmp2,"%6.2fG",traf/GBYTE);	
	    else if (traf > MB) 	sprintf(tmp2,"%6.2fM",traf/MBYTE);
	    else 			sprintf(tmp2,"%6.2fK",traf/KBYTE);
	    strcat(tmp3,tmp2);		
	    sprintf(tmp2,"|");
	    strcat(tmp3,tmp2);		
}


u_long
GetPercent(u_long i)
{
    u_long pr;
	    if (db[i].len > 0) pr = ((double)(db[i].lencur) / (double)(db[i].len)) * H_LINE;
	    else pr=1;

	    if (pr > H_LINE) {	pr=H_LINE; }
    return pr;
}



void
PrintColorString(u_long i)
{
    char nullstring [320];
    u_long  pr = GetPercent(i);
    bzero(&nullstring[0],320);
    strncat(nullstring,tmp3,pr);
    if (db[i].refer != MAXTHREADS) {SETCOLOR(6); }
    else {
    if (V_CURSOR != i)	SETCOLOR(7);	else	SETCOLOR(5);
    }	    
	    wprintw(win,"%s",nullstring);
	    bzero(&nullstring[0],320);
	    strncat(nullstring,tmp3 + pr,H_LINE);
    if (db[i].refer != MAXTHREADS) {SETCOLOR(6); }
    else {
    if (V_CURSOR != i)	SETCOLOR(3);	else	SETCOLOR(8);
    }
    	    wprintw(win,"%s\n",nullstring);
	    SETCOLOR(1);wprintw(win," ");
}


void
CheckMarkDel(u_long i)
{
    if (V_CURSOR == i && DEL_FILE == 1) 
    {
    db[i].fordel = 1; db[i].begin = 0;DEL_FILE=0;IncreaseCursor();
    }
}
