/*-
 * Copyright (c) 2008 Andrey V. Surkov <nsand@sura.ru>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	 $Id: skyfish.c,v 0.52 2008/05/23 07:54:36 nsand Exp $
 */
 
#include <string.h>
#include <errno.h>
#include <signal.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>

#include <sys/ioctl.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <sys/event.h>


#include <netinet/in.h>
#include <netinet/in_systm.h>
#include <netinet/ip.h>
#include <netinet/ip_var.h>
#include <netinet/tcp.h>
#include <netinet/tcp_var.h>

#include <ncurses.h> 

#include "skyfish.h"

#ifndef NTOHS
#define NTOHS(a) (a) = ntohs((a))
#endif

u_long MAXTHREADS;
char 	tmp[LENTMP];
u_char	FAST_SCREEN;			/* fast refresh screen */
u_long FILTERBUFFER;

int done;
char ifaces[MAXIFACE][100];
char *readdev;
struct sigaction action;
struct sigaction chwindow;

u_long speed;
u_long countpks;
u_long discount;
u_long frpks;
struct pks *db;
struct filter *filterdb;
struct filterdbt *dbt;
struct filterdbb *dbb;


int
main(int argc, char **argv)
{
    if (init_scr()) 	return 1;
    if (initialize()) 	return 1;
    if (initnode())  	return 1;
    ShutNodes();
    if (AddNodes()) 	return 1;
    CloseSocket();
    if (CheckPaths()) 	return 1;

    int fd, kq, n=0;
    struct timespec ts = {1, 0};    /*  0      */

    u_long i;
    for (i = 0;i < MAXTHREADS; i++) {
	ClearPKS(i);
    }

    fd = open("/dev/skyfish", O_RDONLY, 0);
    if (fd == -1) {
	printf("Can't open /dev/skyfish \n");
	return 1;
    }

    kq = kqueue();
    if (kq == -1)
	printf("error get kqueue\n");
	      
    struct kevent change;
    struct kevent event;    

    ShowSpeed();
    show_table();
    EV_SET(&change, fd, EVFILT_READ, EV_ADD | EV_ENABLE, 0, 0, NULL);//isok());

    time_t curt = 0, oldt=0;

    int ch;    
    int isevent = 0;
    
        if (RegisterTimer() != 0) 	return 1;
    done = 1;	

while(done)
{
        isevent = kevent(kq, &change, 1, &event, 1, &ts);
	ch=getch();
	if (ch !=  ERR) CheckKey(ch);
	if (FAST_SCREEN) show_table();
	
	time(&curt);
	if ((curt - oldt) >= TIME_CHECK) { 
		oldt = curt; 
		if (!FAST_SCREEN) show_table();
		CheckTime(); 	//move to timer clock
	}
	
	if (isevent > 0) {
	    for (i = 0; i < event.data;i++)  {
		bzero(readdev,READBUFSIZE);
	        if (read(fd, readdev,READBUFSIZE) > 0) {
		    handler(readdev);
		}  else {;}
	    }
	}

    }
    close(kq);	
    close(fd);	
    CloseProg();
    return 0;	
}


/*
*   main handler received packet
*/
int 
handler(const char* packet)
{
    struct ip *ip  = (struct ip *) (packet + ETHSIZE);
    int l = ip->ip_hl<<2;
    u_long sum =(ip->ip_sum);
    ip->ip_sum=0;
    u_short chksum = checksum((const u_char*)(packet + ETHSIZE), l, sum);
    ip->ip_sum=sum;

    u_short chk_sumip = in_cksumip((u_short *)(packet + ETHSIZE),  ip->ip_hl<<2, ip->ip_sum);
    if (chksum != 0)  { 
	discount ++;
	return 1;
    }

    u_long lenip = ntohs(ip->ip_len);
    u_long len_tcp = lenip - (ip->ip_hl<<2);
    speed   += lenip;
    AddAlltraf(lenip);
    u_short ipoff = ntohs(ip->ip_off);
    
    if ((ipoff & IP_OFFMASK) || (ipoff & IP_MF))
    {
//	 if ((ntohs(ip->ip_off) & IP_DF) == 1) {
//    Logger("fragment\n");
//	ClearTMP();
//	sprintf(tmp,"frag id = %-10lu  off = %-10lu   dst= %08x src= %08x\n",ntohs(ip->ip_id),ipoff,ip->ip_dst.s_addr,ip->ip_src.s_addr);
//	Logger(tmp);
//    frpks++;
     return 1;
    }

    if (ip->ip_p != IPPROTO_TCP) return 0;	





/*
u_char *pseudopacket;

pseudoheader.source_addr = ip->ip_src;
pseudoheader.destination_addr = ip->ip_dst;
pseudoheader.zero = 0;
pseudoheader.protocol = IPPROTO_TCP;
//pseudoheader.length = htons(size_tcp + size_payload);
pseudoheader.length = htons(len_tcp);

pseudopacket = (char *)malloc(sizeof(pseudoheader) + len_tcp);
if ( !pseudopacket ) {
perror ("malloc");
exit (1);
}
memcpy (pseudopacket, &pseudoheader, sizeof (pseudoheader));

memcpy (pseudopacket + sizeof (pseudoheader),packet + (ip->ip_hl<<2) + ETHSIZE, len_tcp);
//memcpy (pseudopacket + sizeof (pseudoheader) + len_tcp,packet + size_ethernet + size_ip + size_tcp, size_payload);
*/

    struct tcphdr *tcp	= (struct tcphdr *)(packet + (ip->ip_hl<<2) + ETHSIZE);	

//tcp->th_sum = 0;
//tcp->th_sum = tcp_checksum3 ((u_short *)pseudopacket, sizeof (pseudoheader)+ len_tcp);
/*
    u_short chk_sum3  = tcp_checksum3 ((u_short *)pseudopacket, sizeof (pseudoheader)+ len_tcp);
    if (chk_sum3 != tcp->th_sum) {
	discount++;
	ClearTMP();
	sprintf(tmp," tcp eroor chk_sum3=%x tcp_.th_sum=%x\n",chk_sum3, tcp->th_sum);
	Logger(tmp);
	return 1;
    }

*/

    u_short chk_sum  = tcp_cksum(ip->ip_src, ip->ip_dst, ip->ip_p,len_tcp,(const char *)tcp, tcp->th_sum);
    if (chk_sum != tcp->th_sum && chk_sum != 0 && tcp->th_sum != 0) {
	discount++;
	ClearTMP();
	sprintf(tmp," tcp eroor chk_sum=%x tcp_.th_sum=%x\n",chk_sum, tcp->th_sum);
	Logger(tmp);
	return 1;
    }

    if ((len_tcp - (tcp->th_off<<2)) == 0) return 1;
    NTOHS(ip->ip_len);
    if (ip->ip_p == IPPROTO_TCP) parse_TCP(packet);
return 0;
} 


/*
*  initialise vars from config file
*/
int 
initialize(void)
{
    discount 	= 0;
    countpks 	= 0;
    SetRealspeed(0);
    SetAlltraf(0);
    SetWritraf(0);
    speed 	= 0;
    FAST_SCREEN = 1;
    MAXTHREADS 	= 100;
    FILTERBUFFER 	= 100;
    SetTLB(10);
SetPathLog("./log.txt");
SetPathOk("./ok");
SetPathTmp("./temp");    
SetPathInc("./incomplete");

    filterdb = (struct filter*)malloc(FILTERBUFFER * sizeof(struct filter));
	if (filterdb == NULL) 	{ printf("calloc filterdb error\n"); 	return 1;}

    dbt = (struct filterdbt*)malloc(DBTSIZE * sizeof(struct filterdbt)); // 
	if (dbt == NULL) 	{ printf("calloc dbt error\n"); 	return 1;}

    dbb = (struct filterdbb*)malloc(DBTSIZE * sizeof(struct filterdbb)); // 
	if (dbb == NULL) 	{ printf("calloc dbb error\n"); 	return 1;}

    bzero(dbb,DBTSIZE * sizeof(struct filterdbb));
    int i=0;
    char var[100];
    u_int64_t value;
    FILE *fd;
    if ((fd = fopen("./skyfish.conf","r")) == NULL){
        if ((fd = fopen("/usr/local/etc/skyfish.conf","r")) == NULL){
	    printf("Not found file skyfish.conf\n");
	    return 1;
	}
    }
    int rd = 0;
    int done = 0;
    u_long n;
    bzero(var,100);
    while (done == 0) {
	if (strlen(var) == 0) rd = fscanf(fd,"%s",var);
	    else {printf("error format file skyfish.conf\n"); done=1;exit(1);}
	if (!strcmp(var,"end")) done = 1;
	if (rd = 0) done = 1;
	if (!strcmp(var,"TIME_LIVE_THREAD")) 	{fscanf(fd,"%lu",&n); SetTLT(n); memset(var,0,100);}
	if (!strcmp(var,"TIME_END_FILE")) 	{fscanf(fd,"%lu",&n); SetTEF(n); memset(var,0,100);}
	if (!strcmp(var,"NOCOMPLETE")) 		{fscanf(fd,"%lu",&n);	SetNocomplete(n);	memset(var,0,100);}
	if (!strcmp(var,"MAXTHREADS")) 		{fscanf(fd,"%lu",&MAXTHREADS);		memset(var,0,100);}
	if (!strcmp(var,"OK")) 			{ClearTMP();fscanf(fd,"%s",tmp); SetPathOk(tmp);memset(var,0,100);}
	if (!strcmp(var,"TEMP")) 		{ClearTMP();fscanf(fd,"%s",tmp); SetPathTmp(tmp);memset(var,0,100);}
	if (!strcmp(var,"INCOMPLETE")) 		{ClearTMP();fscanf(fd,"%s",tmp); SetPathInc(tmp);memset(var,0,100);}
	if (!strcmp(var,"version")) 			{memset(var,0,100);   fscanf(fd,"%s",var);	
	if (strcmp(var,VERSION)) {printf("Error skyfish.conf is other version!\n"); return 1;}memset(var,0,100);}
	if (!strcmp(var,"LOG")) 		{ClearTMP();fscanf(fd,"%s",tmp); SetPathLog(tmp);memset(var,0,100);}
	if (!strcmp(var,"iface")) 		{if (Getnumface() < MAXIFACE) fscanf(fd,"%s",ifaces[Getnumface()]);Addnumface();	memset(var,0,100);}	

	if (!strcmp(var,"^t")) {
	    for (i=0; i < DBTSIZE; i++)
		if (strlen(dbt[i].type) == 0) {
		    fscanf(fd,"%s",dbt[i].search); 
		    fscanf(fd,"%s",dbt[i].type);
		    i = DBTSIZE;
		}
	memset(var,0,100);		
	}
	if (!strcmp(var,"^b")) {
	    for (i=0; i < DBTSIZE; i++)
		if (strlen(dbb[i].type) == 0) {
		    fscanf(fd,"%s",dbb[i].search); 
		    fscanf(fd,"%s",dbb[i].type);
//		    printf("%s %s %u\n",dbb[i].search,dbb[i].type,strlen(dbb[i].search));
		    i = DBTSIZE;
		}
	memset(var,0,100);		
	}

	if (!strcmp(var,"type")) {
	    for (i=0; i < FILTERBUFFER; i++)
		if (strlen(filterdb[i].type) == 0) {
		    fscanf(fd,"%s",filterdb[i].type); //printf("type= %s \t",filetdb[i].type);
		    memset(var,0,100);
		    fscanf(fd,"%s",var);
			if (!strcmp(var,"-")) { 
				filterdb[i].min = 0; 
				filterdb[i].max = 0; 
				i = FILTERBUFFER; 
				memset(var,0,100);
				continue;
			} 

		    sscanf(var,"%llu",&filterdb[i].min);

			if (strcasestr(var,"kb")  != NULL) {filterdb[i].min *= KBYTEL;}
			else if (strcasestr(var,"mb")  != NULL) {filterdb[i].min *= MBYTEL;}
			else if (strcasestr(var,"gb")  != NULL) {filterdb[i].min *= GBYTEL;}			
			else if (strcasestr(var,"tb")  != NULL) {filterdb[i].min *= TBYTEL;}
			else { memset(var,0,100);}
		    if (filterdb[i].max == 0) fscanf(fd,"%s",var); 
		    sscanf(var,"%llu",&filterdb[i].max);
			if (strcasestr(var,"kb")  != NULL) {filterdb[i].max *= KBYTEL;memset(var,0,100);}
			else if (strcasestr(var,"mb")  != NULL) {filterdb[i].max *= MBYTEL;bzero(var,100);}
			else if (strcasestr(var,"gb")  != NULL) {filterdb[i].max *= GBYTEL;bzero(var,100);}			
			else if (strcasestr(var,"tb")  != NULL) {filterdb[i].max *= TBYTEL;bzero(var,100);}
		    if (rd = 0) done = 1;			
		    i = FILTERBUFFER;
		    }
	    	
	}
    }
    
    fclose(fd);

    if (OpenLog() != 0) return 1;

    readdev = (char*) malloc(READBUFSIZE);
    if (readdev == NULL) 	{ printf("malloc readdev error\n"); 	return 1;}

    db = (struct pks *)malloc(MAXTHREADS*PKS);
    if (db 	== NULL) 	{ printf("malloc db error\n");  	return 1;}

    TransformDBB();
return 0;
}


u_short
in_cksumip(const u_short *addr, register u_int len, int csum)
{
	int nleft = len;
	const u_short *w = addr;
	u_short answer;
	int sum =-(csum);
	while (nleft > 1)  {
		sum += *w++;
		nleft -= 2;
	}
	if (nleft == 1)
		sum += htons(*(u_char *)w<<8);

	sum = (sum >> 16) + (sum & 0xffff);	/* add hi 16 to low 16 */
	sum += (sum >> 16);			/* add carry */
	answer = ~sum;				/* truncate to 16 bits */
	return answer;
}



/* for computing TCP/UDP checksums, see TCP/IP Illustrated p. 145 */
u_short 
tcp_cksum(const struct in_addr src, const struct in_addr dst,
				  u_char proto, u_short len,const char *hstart, u_short csum)
{
	struct pseudo {
		struct in_addr src;
		struct in_addr dst;        
		u_char zero;
		u_char proto;        
		u_short length;
	};
	struct pseudo hdr;
	hdr.src = src;
	hdr.dst = dst;
	hdr.zero = 0;
	hdr.proto = proto;
	hdr.length = ntohs(len);
	u_short res=0;
	res = in_cksumip((u_short *) &hdr, sizeof(struct pseudo),0);
	return in_cksumip((u_short *) hstart, len,csum + res);
}



void 
CloseProg (void)
{
    free(readdev);
    free(db);
    free(filterdb);
    free(dbt);
    free(dbb);
    initnode();
    ShutNodes();
    CloseFiles();
    CloseWindows();
}


/*
*  switch fast screen
*/
void
ChFastScr(void)
{
    if (FAST_SCREEN == 0) FAST_SCREEN = 1;
    else FAST_SCREEN = 0;
}


/*
*   Handler timer
*/
void 
callme(int sig_no)
{
    SetRealspeed(speed);
    speed=0;
    ShowSpeed();
    alarm(INTERVAL);
}


/*
*  register real timer for show title programm on the screen
*/
int
RegisterTimer(void)
{
    sigemptyset(&action.sa_mask);
    sigemptyset(&chwindow.sa_mask);
    action.sa_handler = (void (*)(int))callme;
    action.sa_flags = SA_RESTART;
    chwindow.sa_handler = (void (*)(int))Resize;
    chwindow.sa_flags = SA_RESTART;
    
    if (sigaction(SIGALRM, &action, 0) == -1)
    {
    printf("error sigaction\n");
    return 1;
    }

    if (sigaction(SIGWINCH, &chwindow, 0) == -1)
    {
    printf("error SIGWINCH action\n");
    return 1;
    }
    
    if (alarm(INTERVAL) == -1) {
    printf("error sigalarm\n");
	return 1;
    }
return 0;
}


inline u_short 
checksum(const u_char* data, int len, u_long c)
{
	while(len > 1)
	{	c += *(u_short*)data;
		data += 2; len -= 2;
	}
	if(len) c += *data;

	c = (c >> 16) + (c & 0xffff);
	c += (c >> 16);
	
	return (u_short)(~c);
}

void
TransformDBB(void)
{
    u_long i;
    u_char X;
    u_long n;
    u_long m=0;
    char * ps;
    char * uu;
    u_char tt[100];

    for (i=0; i < DBTSIZE; i++)
	if (strlen(dbb[i].type) > 0) {
	    bzero(tt,100);
	    uu = dbb[i].search;
	    m=0;
	    for (n=0; n<50 && *uu>0; n++) {
		if (*uu == '\\')
		    {
		    uu++;
		sscanf(uu,"%x",&X); 
		tt[m]=X;	    
		m++;
		uu+=2;
		} else {
		tt[m]=*uu;
		m++;
		uu++;
		     }
	    }
	dbb[i].len = m;
	for (m=0; m<dbb[i].len; m++)
	    dbb[i].search[m] = tt[m];
//	    printf("net type=%s",dbb[i].search);
	}

}



inline u_short tcp_checksum3(u_short *addr, int len)
{
    register int nleft = len;
    register u_short *w = addr;
    register int sum = 0;
    u_short answer = 0;

     /* Our algorithm is simple, using a 32 bit accumulator (sum), we add
      * sequential 16 bit words to it, and at the end, fold back all the
      * carry bits from the top 16 bits into the lower 16 bits. */

     while (nleft > 1)  {
         sum += *w++;
         nleft -= 2;
     }

     /* mop up an odd byte, if necessary */
     if (nleft == 1) {
         *(u_char *)(&answer) = *(u_char *) w;
         sum += answer;
     }

     /* add back carry outs from top 16 bits to low 16 bits */
     sum = (sum >> 16) + (sum & 0xffff); /* add hi 16 to low 16 */
     sum += (sum >> 16);                /* add carry */
     answer = ~sum;                     /* truncate to 16 bits */
     return(answer);
}
/*
struct _pseudoheader {
struct in_addr source_addr;
struct in_addr destination_addr;
u_char zero; 
u_char protocol;
u_short length;
} pseudoheader;
*/
/*
u_char *pseudopacket;

pseudoheader.source_addr = iph->ip_src;
pseudoheader.destination_addr = iph->ip_dst;
pseudoheader.zero = 0;
pseudoheader.protocol = IPPROTO_TCP;
pseudoheader.length = htons(size_tcp + size_payload);

pseudopacket = (char *)malloc(sizeof(pseudoheader) + sizeof(struct tcphdr) + size_payload);
if ( !pseudopacket ) {
perror ("malloc");
exit (1);
}
memcpy (pseudopacket, &pseudoheader, sizeof (pseudoheader));

memcpy (pseudopacket + sizeof (pseudoheader),packet + size_ethernet + size_ip, size_tcp);
memcpy (pseudopacket + sizeof (pseudoheader) + size_tcp,packet + size_ethernet + size_ip + size_tcp, size_payload);


tcph->th_sum = 0;
tcph->th_sum = tcp_checksum ((u_short *)pseudopacket, sizeof (pseudoheader)+ size_tcp + size_payload);
*/

