/*-
 * Copyright (c) 2001-2004 Roman V. Palagin <romanp@unshadow.net>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	 $Id: ng_skyfish.c,v 0.61 2005/05/23 07:54:36 nsand Exp $
 * 	with changes from Surkov Andrey
 */
 
 
#include <sys/param.h>
#include <sys/systm.h>
#include <sys/kernel.h>
#include <sys/mbuf.h>
#include <sys/malloc.h>
#include <sys/ctype.h>
#include <sys/errno.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <sys/socketvar.h>
#include <sys/ucred.h>
#include <sys/sysctl.h>
#include <sys/proc.h>

#include <net/bpf.h>
#include <net/ethernet.h>
#include <net/route.h>

#include <netinet/in_systm.h>
#include <netinet/in.h>
#include <netinet/in_pcb.h>
#include <netinet/ip.h>
#include <netinet/ip_icmp.h>
#include <netinet/ip_var.h>
#include <netinet/tcp.h>
#include <netinet/tcp_var.h>
#include <netinet/udp.h>
#include <netinet/udp_var.h>

#include <netgraph/ng_message.h>
#include <netgraph/ng_parse.h>
#include <netgraph/netgraph.h>

#include <sys/conf.h>   /* cdevsw struct */
#include <sys/uio.h>    /* uio struct */
#include <sys/malloc.h>

#include "ng_skyfish.h"
#include "drv.h"

/*
 * FreeBSD version check
 */
#if !defined(__FreeBSD_version) || \
		((__FreeBSD_version < 600000))
#error "Module not supported on this version of FreeBSD."
#endif

#ifndef NTOHS
#define NTOHS(a) (a) = ntohs((a))
#endif

static ng_constructor_t	ng_skyfish_constructor;
static ng_shutdown_t	ng_skyfish_shutdown;
static ng_newhook_t	ng_skyfish_newhook;
static ng_rcvdata_t	ng_skyfish_rcvdata;
static ng_disconnect_t	ng_skyfish_disconnect;
static int ng_skyfish_mod_event(module_t mod, int event, void *data);

static struct ng_type ng_skyfish_typestruct = {
	.version = NG_ABI_VERSION,
	.name = NG_SKYFISH_NODE_TYPE,
	.mod_event = ng_skyfish_mod_event,
	.constructor = ng_skyfish_constructor,
	.shutdown = ng_skyfish_shutdown,
	.newhook = ng_skyfish_newhook,
	.rcvdata = ng_skyfish_rcvdata,
	.disconnect = ng_skyfish_disconnect,
};

NETGRAPH_INIT(skyfish, &ng_skyfish_typestruct);

struct skyfish_hook {
	hook_p  		hook;
	node_p			node;
	struct mtx      	h_mtx;         /* protects hook data */
};

typedef struct skyfish_hook *hinfo_p;

static int  fifo_add(struct mbuf *);

static int
ng_skyfish_mod_event(module_t mod, int event, void *data)
{
	switch(event) {
	case MOD_LOAD:
		break;
	case MOD_UNLOAD:
		break;		
	}

	return (0); 
}

static int
ng_skyfish_constructor(node_p nodep)
{
	return (0);
}

static int
ng_skyfish_newhook(node_p node, hook_p hook, const char *name) {
    printf("ng_skyfish_newhook:%s\n",name);
	hinfo_p		hip;
	MALLOC(hip, hinfo_p, sizeof(*hip), M_NETGRAPH, M_NOWAIT | M_ZERO);
	if (hip == NULL)
		return (ENOMEM);

	hip->hook = hook;
	hip->node = node;
    mtx_init(&hip->h_mtx, "ng_skyfish hash", "", MTX_DEF);
    
	NG_HOOK_SET_PRIVATE(hook, hip);
	return (0);
}

static int
ng_skyfish_rcvdata(hook_p hook, item_p item) {

	//const hinfo_p 	hip = NG_HOOK_PRIVATE(hook);
	critical_enter();
	int error = 0;
	struct mbuf *m;
	rcvdata ++;
	m = NGI_M(item);
		struct ether_header *eh;
		eh = mtod(m, struct ether_header *);
		NTOHS(eh->ether_type);
		if (eh->ether_type == ETHERTYPE_IP) 
		    {
		    error = fifo_add(m);
		    }
	NG_FREE_ITEM(item);
	critical_exit();
	return (error);
}

static int
ng_skyfish_shutdown(node_p node)
{
	NG_NODE_UNREF(node);
	return (0);
}

static int
ng_skyfish_disconnect(hook_p hook)
{
	const hinfo_p hip = NG_HOOK_PRIVATE(hook);

	NG_HOOK_SET_PRIVATE(hook, NULL);
	mtx_destroy(&hip->h_mtx);
	FREE(hip, M_NETGRAPH);
	if ((NG_NODE_NUMHOOKS(NG_HOOK_NODE(hook)) == 0) &&
		(NG_NODE_IS_VALID(NG_HOOK_NODE(hook)))) {
		ng_rmnode_self(NG_HOOK_NODE(hook));
	}

	return (0);
}

/*
 * Add streams into FIFO buffer in kernel 
 * 
 */
static int
fifo_add(struct mbuf * m) {

    rcvdata2++;
    u_char *data;
    uint32_t length = m_length(m,NULL);
    uint32_t len = m->m_len;
    
    if (len != length) 	{
      return 1;}
    if (len == 0 ) 	{
        return 1;}

    data = mtod(m, u_char*);
    pos_f = pos_w + len + 4;	/* future position */

    if (pos_w < pos_r && pos_f >= pos_r) {
    		printf("overflow 1\n");
	return 1;
    }

    if (pos_w > pos_r)
	if (pos_f >= (BUFFERSIZE - 4))
	    if (len >= pos_r) {
		printf("overflow 2\n");
		return 1;
	    }
    if (pos_f >= (BUFFERSIZE - 4)) {
	memcpy(fifobuf + pos_w, &len, 4);
	pos_w=0;   
    }

    memcpy(fifobuf + pos_w, &len, 4);
    memcpy(fifobuf + pos_w + 4, data, len);
    pos_w += (len + 4);
    isdata++;
    KNOTE_UNLOCKED(&echo_rsel.si_note, 0);				    
    return 0;
}
